import sys
from typing import Dict

from rich import print

warned: Dict[str, bool] = {}


def check(name):
    if name in warned:
        return
    warned[name] = True
    print(  # noqa: T201
        f"[bold yellow]Using the enterprise {name} feature requires a license, unless used for non-commerical use."
        "Please contact us at contact@solara.dev to get a license.",
        file=sys.stderr,
    )



import json
from pathlib import Path
from typing import Any, List

import solara
from rich import print as rprint
from solara.server import settings
from typing_extensions import TypedDict

from .. import license


class DocumentData(TypedDict):
    id: str
    text: str
    location: str
    title: str


def build_index(base_url: str):
    license.check("search")
    import solara.server.app
    import solara.server.kernel

    build_path = settings.ssg.build_path
    assert build_path is not None
    app_script = solara.server.app.apps["__default__"]
    rprint(f"Building search index for {app_script.name} by using {build_path}")
    routes = app_script.routes

    documents: List[DocumentData] = []

    for route in routes:
        add_document(f"{base_url}/", route, build_path, documents)
    index_file = app_script.directory.parent / "assets" / "search.json"
    rprint("Writing search index to", index_file)
    with open(index_file, "w") as f:
        json.dump(documents, f, indent=4)


def add_document(base_url: str, route: solara.Route, build_path: Path, documents: List[DocumentData]):
    url = base_url + (route.path if route.path != "/" else "")
    if not route.children:
        path = build_path / ("index.html" if route.path == "/" else route.path + ".html")
        rprint("Processing", path)
        from bs4 import BeautifulSoup  # , Tag

        if not path.exists():
            rprint(f"Warning: {path} does not exist")
        else:
            soup = BeautifulSoup(path.read_text("utf8"), "html.parser")
            node = soup.find(class_="solara-page-content-search")

            if node is None:
                rprint(f"Warning: {path} has no solara-page-content-search")
            else:
                # split by h1 and h2
                parts: List[List[Any]] = [[]]
                ids = [None]
                titles = [soup.title.string if soup.title else ""]
                current = parts[-1]
                # remove invisible title elements
                for el in node.find_all("span", attrs={"style": "display: none;"}):
                    el.string = ""
                for el in node.descendants:
                    if el.name == "h1" or el.name == "h2":
                        ids.append(el.get("id"))
                        parts.append([])
                        titles.append(el.string)
                        current = parts[-1]
                    else:
                        current.append(el)
                # join the next node
                text = ""
                for i, part in enumerate(parts):
                    for el in part:
                        if el.string:
                            text += el.string.strip() + " "
                    # texts.append("")
                    id = ids[i]
                    location = f"{url}#{id}" if id else url
                    text = text.strip()
                    title = titles[i].strip()
                    if title != text:
                        documents.append({"id": id, "text": text, "location": location, "title": title})
                    text = ""
                # documents.append({"id": url, "html": str(node), "location": url})
    for child in route.children:
        add_document(url + "/", child, build_path / Path(route.path), documents)



import logging
import urllib.parse
from typing import Optional

from solara.routing import router_context
from solara.server import settings

logger = logging.getLogger("solara-enterprise.auth")


def get_logout_url(return_to_path: Optional[str] = None):
    if return_to_path is None:
        router = router_context.get()
        return_to_path = router.path
    if return_to_path.startswith("/"):
        return_to_path = return_to_path[1:]
    assert settings.main.base_url is not None
    return_to_app = urllib.parse.quote(settings.main.base_url + return_to_path)
    return_to = urllib.parse.quote(settings.main.base_url + f"_solara/auth/logout?redirect_uri={return_to_app}")
    client_id = settings.oauth.client_id
    url = f"https://{settings.oauth.api_base_url}/{settings.oauth.logout_path}"
    if settings.oauth.logout_path.startswith("http"):
        url = settings.oauth.logout_path
    return f"{url}?returnTo={return_to}&redirect_uri={return_to}&post_logout_redirect_uri={return_to}&client_id={client_id}"


def get_login_url(return_to_path: Optional[str] = None):
    if return_to_path is None:
        router = router_context.get()
        return_to_path = router.path
    if return_to_path.startswith("/"):
        return_to_path = return_to_path[1:]
    assert settings.main.base_url is not None
    redirect_uri = urllib.parse.quote(settings.main.base_url + return_to_path)
    return f"/_solara/auth/login?redirect_uri={redirect_uri}"



from typing import Dict, Optional, cast

from solara.lab import Reactive

from .components import Avatar, AvatarMenu
from .utils import get_login_url, get_logout_url

__all__ = ["user", "get_login_url", "get_logout_url", "Avatar", "AvatarMenu"]

user = Reactive(cast(Optional[Dict], None))
